structure Interface : INTERFACE = Interface();

structure NL_LrVals : NL__LRVALS =
   NL_LrValsFun(structure Token = LrParser.Token);

structure NL_Lex : LEXER =
   NL_LexFun(structure Tokens = NL_LrVals.Tokens
             structure Interface = Interface);

structure NL_Parser : PARSER =
   Join(structure ParserData = NL_LrVals.ParserData
        structure Lex = NL_Lex
        structure LrParser = LrParser);

signature PARSE =
  sig
    val parse : unit -> (NL_Parser.svalue,NL_Parser.pos) LrParser.Token.token
    exception Undefined
  end  

functor Parse (structure Interface : INTERFACE
               structure Parser : PARSER
                  sharing type Parser.arg = Interface.arg
                  sharing type Parser.pos = Interface.pos
               structure Tokens : NL__TOKENS
                  sharing type Tokens.token = Parser.Token.token
                  sharing type Tokens.svalue = Parser.svalue
               ) : PARSE =
  struct
    exception Undefined

    fun invoke lexstream =
        let val print_error = 
                fn (s,i:int,_) =>
                   TextIO.output(TextIO.stdOut,"Error, line " 
                                               ^ (Int.toString i) 
                                               ^ ", " ^ s ^ "\n")
        in 
          NL_Parser.parse(0,lexstream,print_error,())
        end

    fun parse () =
        let 
          fun reader (n:int) = 
              case (TextIO.inputLine TextIO.stdIn) of
                SOME s => s 
              | NONE => raise Undefined

          (* Fr SML 110.0.3 stattdessen:
             fun reader (n:int) = TextIO.inputLine TextIO.stdIn *)

          val dummyEOF = NL_LrVals.Tokens.EOF(0,0)

          fun loop lexer =
              let val (result,lexer) = invoke lexer
                  val (nextToken,lexer) = NL_Parser.Stream.get lexer
                  val _ = case result of 
                           (SOME (_,s)) => 
                              TextIO.output(TextIO.stdOut, 
                                            "Value is: " ^ s ^ "\n")
                          | NONE => 
                              TextIO.output(TextIO.stdOut, 
                               "Value is neither a Boolean nor an object.\n")
              in if NL_Parser.sameToken(nextToken,dummyEOF) 
                 then nextToken
                 else loop lexer
              end
        in
          loop (NL_Parser.makeLexer reader)
        end
  end
        
structure NL : PARSE =
   Parse (structure Interface = Interface
          structure Parser = NL_Parser
          structure Tokens = NL_LrVals.Tokens );

